#!/usr/bin/python

import sys
import os
import json 

#print("########################PARSERESULTS.PY########################")

GenScores = []
SimScores = []
TrigSimScores = []
RecoScores = []
TotalScores = []

MaxGen = 0
AvgGen = 0
MedGen = 0
MinGen = 1000000

MaxSim = 0
AvgSim = 0
MedSim = 0
MinSim = 1000000

MaxTrigSim = 0
AvgTrigSim = 0
MedTrigSim = 0
MinTrigSim = 1000000

MaxReco = 0
AvgReco = 0
MedReco = 0
MinReco = 1000000

MaxTotal = 0
AvgTotal = 0
MedTotal = 0
MinTotal = 1000000

# Each copy generates an output file
for i in range(0, int(os.environ['NCOPIES'])):
	filename = "proc_" + str(i + 1) + "/output"

	# The parser reports separately the time taken by event generation, simulation, trigger simulation, 
	# and reconstruction (and the total). Times are reported from many sub-modules; these keywords 
	# are used to make the groupings
	keywords = ["EvtGenInput", "Sum_Simulation", "Sum_TriggerSimulation", "SoftwareTrigger"]
	keepgoing = False;
	startintwo = False;

	Gen = 0
	Sim = 0
	TrigSim = 0
	Reco = 0
	Total = 0
	temptotal = 0

	with open(filename) as out:
		for line in out:

			# Flag to start parsing, initially false
			if keepgoing:
				splitline = line.split("|")
				#print splitline

				# Lines with Sum_ are the total time taken by the executable, not just
				# event processing calls
				if "Sum_" not in line:
					#print splitline
					temptotal = temptotal + float(splitline[3].strip())

				if splitline[0].strip() in keywords:
					if splitline[0].strip() == "EvtGenInput":
						Gen = float(splitline[3].strip())
						temptotal = 0
					elif splitline[0].strip() == "Sum_Simulation":
						Sim = temptotal
						temptotal = 0
					elif splitline[0].strip() == "Sum_TriggerSimulation":
						TrigSim = temptotal
						temptotal = 0
					elif splitline[0].strip() == "SoftwareTrigger":
						Reco = temptotal
						temptotal = 0
						keepgoing = False
			
				#print splitline[0].strip() + ": " + splitline[3].strip()

			# Almost there...
			if startintwo:
				keepgoing = True
				startintwo = False

			# This line is two lines before the part we care about, and is more distinctive
			# than the next line
			if "Name" in line and "Calls" in line:
				startintwo = True
				#print line	

            
			if "Total" in line: 
				if "[INFO] [TRGECLModule]" in line:
					#This line appears now and parsing is fine if we pass
					print ("Skipping bad 'Total' line")
				else:	
					try:
						Total = float(line.split("|")[3].strip())
					except:
						print ("ERROR! New line with string 'Total', unexpected behaviour! Line: '%s'" % line)
						raise
				
			

	# Output is in seconds; convert to events / second
	Gen = float(os.environ['NEVENTS_THREAD']) / Gen
	Sim = float(os.environ['NEVENTS_THREAD']) / Sim
	TrigSim = float(os.environ['NEVENTS_THREAD']) / TrigSim
	Reco = float(os.environ['NEVENTS_THREAD']) / Reco
	Total = float(os.environ['NEVENTS_THREAD']) / Total

	#print "Gen: " + str(Gen)
	#print "Sim: " + str(Sim)
	#print "TrigSim: " + str(TrigSim)
	#print "Reco: " + str(Reco)
	#print "Total: " + str(Total)

	GenScores.append(Gen)
	SimScores.append(Sim)
	TrigSimScores.append(TrigSim)
	RecoScores.append(Reco)
	TotalScores.append(Total)

	#print GenScores
	#print SimScores
	#print TrigSimScores
	#print RecoScores
	#print TotalScores

	if Gen > MaxGen:
		MaxGen = Gen
	if Sim > MaxSim:
		MaxSim = Sim
	if TrigSim > MaxTrigSim:
		MaxTrigSim = TrigSim
	if Reco > MaxReco:
		MaxReco = Reco
	if Total > MaxTotal:
		MaxTotal = Total

	if Gen < MinGen:
		MinGen = Gen
	if Sim < MinSim:
		MinSim = Sim
	if TrigSim < MinTrigSim:
		MinTrigSim = TrigSim
	if Reco < MinReco:
		MinReco = Reco
	if Total < MinTotal:
		MinTotal = Total

	AvgGen = AvgGen + Gen / float(os.environ['NCOPIES'])
	AvgSim = AvgSim + Sim / float(os.environ['NCOPIES'])
	AvgTrigSim = AvgTrigSim + TrigSim / float(os.environ['NCOPIES'])
	AvgReco = AvgReco + Reco / float(os.environ['NCOPIES'])
	AvgTotal = AvgTotal + Total / float(os.environ['NCOPIES'])


GenScores.sort()
SimScores.sort()
TrigSimScores.sort()
RecoScores.sort()
TotalScores.sort()

if len(GenScores) % 2 == 0:
        MedGen = (GenScores[int(len(GenScores) / 2)] + GenScores[int(len(GenScores) / 2 - 1)]) / 2.0
        MedSim = (SimScores[int(len(SimScores) / 2)] + SimScores[int(len(SimScores) / 2 - 1)]) / 2.0
        MedTrigSim = (TrigSimScores[int(len(TrigSimScores) / 2)] + TrigSimScores[int(len(TrigSimScores) / 2 - 1)]) / 2.0
        MedReco = (RecoScores[int(len(RecoScores) / 2)] + RecoScores[int(len(RecoScores) / 2 - 1)]) / 2.0
        MedTotal = (TotalScores[int(len(TotalScores) / 2)] + TotalScores[int(len(TotalScores) / 2 - 1)]) / 2.0
else:
        MedGen = GenScores[int(len(GenScores) / 2)]
        MedSim = SimScores[int(len(SimScores) / 2)]
        MedTrigSim = TrigSimScores[int(len(TrigSimScores) / 2)]
        MedReco = RecoScores[int(len(RecoScores) / 2)]
        MedTotal = TotalScores[int(len(TotalScores) / 2)]


OutputJSON = {}
OutputJSON['run_info'] = {}
OutputJSON['report'] = {}
OutputJSON['report']['wl-scores'] = {}
OutputJSON['report']['wl-stats'] = {}
OutputJSON['report']['wl-stats']['gen'] = {}
OutputJSON['report']['wl-stats']['sim'] = {}
OutputJSON['report']['wl-stats']['reco'] = {}
OutputJSON['report']['wl-stats']['gen-sim-reco'] = {}
OutputJSON['app'] = {}

OutputJSON['run_info']['copies'] = int(os.environ['NCOPIES'])
OutputJSON['run_info']['threads_per_copy'] = int(os.environ['NTHREADS'])
OutputJSON['run_info']['events_per_thread'] = int(os.environ['NEVENTS_THREAD'])
OutputJSON['report']['wl-scores']['gen'] = AvgGen * int(os.environ['NCOPIES'])
OutputJSON['report']['wl-scores']['sim'] = 1.0 / (1.0 / AvgSim + 1.0 / AvgTrigSim) * int(os.environ['NCOPIES'])
OutputJSON['report']['wl-scores']['reco'] = AvgReco * int(os.environ['NCOPIES'])
OutputJSON['report']['wl-scores']['gen-sim-reco'] = AvgTotal * int(os.environ['NCOPIES'])

OutputJSON['report']['wl-stats']['gen-sim-reco']['avg'] = AvgTotal
OutputJSON['report']['wl-stats']['gen-sim-reco']['median'] = MedTotal
OutputJSON['report']['wl-stats']['gen-sim-reco']['min'] = MinTotal
OutputJSON['report']['wl-stats']['gen-sim-reco']['max'] = MaxTotal
OutputJSON['report']['wl-stats']['gen-sim-reco']['count'] = int(os.environ['NCOPIES'])
OutputJSON['report']['wl-stats']['gen']['avg'] = AvgGen
OutputJSON['report']['wl-stats']['gen']['median'] = MedGen
OutputJSON['report']['wl-stats']['gen']['min'] = MinGen
OutputJSON['report']['wl-stats']['gen']['max'] = MaxGen
OutputJSON['report']['wl-stats']['gen']['count'] = int(os.environ['NCOPIES'])
OutputJSON['report']['wl-stats']['sim']['avg'] = 1.0 / (1.0 / AvgSim + 1.0 / AvgTrigSim)
OutputJSON['report']['wl-stats']['sim']['median'] = 1.0 / (1.0 / MedSim + 1.0 / MedTrigSim)
OutputJSON['report']['wl-stats']['sim']['min'] = 1.0 / (1.0 / MinSim + 1.0 / MinTrigSim)
OutputJSON['report']['wl-stats']['sim']['max'] = 1.0 / (1.0 / MaxSim + 1.0 / MaxTrigSim)
OutputJSON['report']['wl-stats']['sim']['count'] = int(os.environ['NCOPIES'])
OutputJSON['report']['wl-stats']['reco']['avg'] = AvgReco
OutputJSON['report']['wl-stats']['reco']['median'] = MedReco
OutputJSON['report']['wl-stats']['reco']['min'] = MinReco
OutputJSON['report']['wl-stats']['reco']['max'] = MaxReco
OutputJSON['report']['wl-stats']['reco']['count'] = int(os.environ['NCOPIES'])

# Borrowed from atlas/gen/atlas-gen/parseResults.py 
try:       
	with open(os.path.join(os.environ['BMKDIR'],"version.json")) as f:         
		version_json = json.loads(f.read())         
		version_json['containment'] = os.environ['flavor']
		OutputJSON["app"] = version_json
except:       
	OutputJSON["app"] = ''


# 2021-11-20 After moving parser logic into parser-driver
# only the WL specific stats are expected
# therefore the log is not added 
#OutputJSON['report']['log'] = os.environ['s_msg']

basewdir = os.environ['baseWDir']
OutputFile = open(basewdir+"/parser_output.json", "w")
json.dump(OutputJSON['report'], OutputFile)
OutputFile.close()
